@@ -87,7 +87,6 @@ impl From<u32> for HostingResult {
 }
 
 impl From<i32> for HostingResult {
-    #[allow(clippy::cast_sign_loss)]
     fn from(code: i32) -> Self {
         Self::from(code as u32)
     }
@@ -320,7 +319,7 @@ pub enum HostingError {
     #[display(fmt = "Initialization of the `hostpolicy` library failed.")]
     LibHostInitFailure,
 
-    // Error only present in `error_codes.h` not in hostf sefs
+    // Error only present in `error_codes.h` not in `host-error-codes.md`
     #[doc(hidden)]
     #[display(fmt = "LibHostExecModeFailure")]
     LibHostExecModeFailure,
