@@ -9,7 +9,7 @@ use std::{
 use super::{NulError, PdCStrInner, PdCStringInner, PdUChar};
 
 /// A platform-dependent c-like string type for interacting with the `hostfxr` and `nethost` API.
-#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Clone)]
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Default)]
 #[repr(transparent)]
 pub struct PdCString(pub(crate) PdCStringInner);
 
@@ -66,12 +66,6 @@ impl DerefMut for PdCString {
     }
 }
 
-impl Default for PdCString {
-    fn default() -> Self {
-        Self(PdCStringInner::default())
-    }
-}
-
 impl Display for PdCStr {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         self.0.fmt(f)
